@@ -136,6 +136,7 @@ class GDebiKDE(GDebiCommon, GDebiKDEDialog):
         self.tabWidget2.setTabText(2,_("Included Files"))
         self.cancelButton.setText(__("kdelibs","&Cancel"))
         self.installButton.setText(_("&Install Package"))
+        self.downloadButton.setText(_("&Download Package"))
         self.DetailsVersionLabel.setText(_("<b>Version:</b>"))
         self.DetailsMaintainerLabel.setText(_("<b>Maintainer:</b>"))
         self.DetailsPriorityLabel.setText(_("<b>Priority:</b>"))
@@ -145,6 +146,7 @@ class GDebiKDE(GDebiCommon, GDebiKDEDialog):
         self.setDisabled(True)
         self.PackageProgressBar.setEnabled(True)
         self.detailsButton.hide()
+        self.downloadButton.hide()
         self.installButton.setIcon(KIcon("dialog-ok"))
         self.cancelButton.setIcon(KIcon("dialog-cancel"))
         self.show()
@@ -168,11 +170,12 @@ class GDebiKDE(GDebiCommon, GDebiKDEDialog):
         self.PackageProgressBar.hide()
         self.connect(self.cancelButton, SIGNAL("clicked()"), self.cancelButtonClicked)
         self.connect(self.installButton, SIGNAL("clicked()"), self.installButtonClicked)
+        self.connect(self.downloadButton, SIGNAL("clicked()"), self.downloadButtonClicked)
         self.connect(self.detailsButton, SIGNAL("clicked()"), self.detailsButtonClicked)
 
-    def open(self, file):
+    def open(self, file, downloaded=False):
         """ load the .deb file """
-        res = GDebiCommon.open(self, file)
+        res = GDebiCommon.open(self, file, downloaded)
         if res == False:
             KMessageBox.error(self, '<b>' + self.error_header + '</b><br>' + self.error_body, self.error_header)
             return False
@@ -223,9 +226,21 @@ class GDebiKDE(GDebiCommon, GDebiKDEDialog):
             self.detailsButton.hide()
             return False
 
+        # check provides
+        provides = self.compareProvides()
+        if provides:
+            self.installButton.setEnabled(False)
+            self.textLabel1_3_2.setText(utf8("<font color=\"#ff0000\"> Error: no longer provides " + ", ".join(provides) + "</font>"))    
+            return False
+
         # set version_info_{msg,title} strings
         self.compareDebWithCache()
 
+        version_status = self._deb.compare_to_version_in_cache(use_installed=False)
+        if (version_status in (DebPackage.VERSION_SAME, DebPackage.VERSION_OUTDATED)):
+            if not self._deb.downloaded:
+                self.downloadButton.show()
+
         if self._deb.compare_to_version_in_cache() == DebPackage.VERSION_SAME:
             #self.textLabel1_3_2.setText(_("Same version is already installed"))
             self.installButton.setText(_("&Reinstall Package"))
@@ -308,7 +323,7 @@ class GDebiKDE(GDebiCommon, GDebiKDEDialog):
         if len(self.install) > 0 or len(self.remove) > 0:
             #if not self.acquire_lock():
             if not self.acquire_lock():
-                #self.show_alert(gtk.MESSAGE_ERROR, self.error_header, self.error_body)
+                #self.show_alert(Gtk.MessageType.ERROR, self.error_header, self.error_body)
                 KMessageBox.sorry(self, '<b>' + self.error_header + '</b><br>' + self.error_body, self.error_header)
                 return False
             fprogress = KDEFetchProgressAdapter(self.installDialog.installationProgress,
@@ -369,6 +384,10 @@ class GDebiKDE(GDebiCommon, GDebiKDEDialog):
             self.errorReport = KMessageBox.error(self, header + text, header)
             sys.exit(1)
 
+    def downloadButtonClicked(self):
+        if self.download_package():
+            self.downloadButton.hide()
+
 class GDebiKDEInstall(QDialog):
     def __init__(self, parent):
         QDialog.__init__(self, parent)
